{-# LANGUAGE UndecidableInstances, FlexibleInstances, MultiParamTypeClasses  #-}

{-| Module  : FiniteCategories
Description : Free subcategory generated by a subset of morphisms of a category C.
Copyright   : Guillaume Sabbagh 2021
License     : GPL-3
Maintainer  : guillaumesabbagh@protonmail.com
Stability   : experimental
Portability : portable

Free subcategory generated by a subset of morphisms of a category @C@.
-}
module Subcategories.FreeSubcategory
(
FreeSubcategory(..)
)
where
    import FiniteCategory.FiniteCategory
    import Data.List (nub)
    import Utils.SetList
    
    -- | The free subcategory generated by a subset of morphisms of a category @C@.
    data FreeSubcategory c m o = FreeSubcategory c [m]
    
    -- | Compose a list of morphisms with generator morphisms to generated new morphisms.
    composeOnce :: (Morphism m o, Eq m, Eq o) => [m] -> [m] -> [m]
    composeOnce m g = nub [m2 @ m1| m1 <- m, m2 <- g, (target m1) == (source m2)]
    
    -- | Compose a list of generator morphisms until it's useless.
    composeUntilEnd :: (Morphism m o, Eq m, Eq o) => [m] -> [m]
    composeUntilEnd g = composeRecursive g g
        where composeRecursive ms g
                | doubleInclusion nextStep ms = ms
                | otherwise = composeRecursive nextStep g
                where
                    nextStep = composeOnce ms g
    allArrows :: (FiniteCategory c m o, Morphism m o, Eq m, Eq o) => FreeSubcategory c m o -> [m]
    allArrows sc@(FreeSubcategory c morphs) = composeUntilEnd $ nub $ morphs++(identities sc)
    
    instance (FiniteCategory c m o, Morphism m o, Eq m, Eq o) => FiniteCategory (FreeSubcategory c m o) m o where
        ob (FreeSubcategory _ morphs) = nub $ [source m | m <- morphs]++[target m | m <- morphs]
        identity (FreeSubcategory c morphs) obj = identity c obj
        ar sc s t = filter (\x -> (source x) == s && (target x) == t) (allArrows sc)
        arrows = allArrows
        
    instance (FiniteCategory c m o, Morphism m o, Eq m, Eq o) => GeneratedFiniteCategory (FreeSubcategory c m o) m o where
        genAr (FreeSubcategory _ morphs) s t = filter (\x -> (source x) == s && (target x) == t) morphs
        decompose = bruteForceDecompose
        genArrows (FreeSubcategory _ morphs) = morphs
        
